#include "ser.h"

#ifdef NOSERIALIZE
inline unsigned long gethex(char* src)
{
  unsigned long number;
  char buf[9];
  strncpy(buf,src,8);
  buf[8]=0;
  sscanf(buf,"%lx",&number);
  return number;
};
#endif

unsigned long findelem(string name,unsigned long pos,char* c,char* w,char* t)
{
  unsigned long tpos=gethex(c+pos+1);
  pos+=10;
  tpos+=33;
  while (t[tpos]!=' ')
    {
      if (!strncmp(t+tpos,name+" ",name.len+1)) return pos;
      pos+=10;
      tpos+=33;
    };
  return 0xffffffff;
}

unsigned long tagcount;
unsigned long tagpos;
void findtag(string name,unsigned long count,unsigned long pos,
             char* c,char* w,char* t)
{
  if (tagcount==count) return;

  /* find gi and compare */
  unsigned long gipos=findelem("gi",pos,c,w,t);
  if (gipos==0xffffffff) return;
  if (c[gipos]!='s') return;
  unsigned long number=gethex(c+gipos+1);
  if (number==0xffffffff) return;
  unsigned long len=gethex(w+number);
  char* s=(char*) malloc(len+1);
  if (!s) log(LOG_FATAL,__FILE__,__LINE__,ERROR_OUT_OF_MEM,0);
  strncpy(s,w+number+8,len);
  s[len]=0;
  if (!strcmp(s,name))
    {
      tagcount++;
      if (tagcount==count)
        {
          free(s);
          tagpos=pos;
          return;
        };
    };
  free(s);

  /* recurse into content */
  unsigned long cpos=findelem("content",pos,c,w,t);
  if (cpos==0xffffffff) return;
  if (c[cpos]!='p') return;
  cpos=gethex(c+cpos+1);
  if (cpos!=0xffffffff) findtag(name,count,cpos,c,w,t);

  if (tagcount==count) return;

  /* recurse into next */
  unsigned long npos=findelem("next",pos,c,w,t);
  if (npos==0xffffffff) return;
  if (c[npos]!='p') return;
  npos=gethex(c+npos+1);
  if (npos!=0xffffffff) findtag(name,count,npos,c,w,t);

  return;
}

bool getpos(char* c,char* w,char* t,
	    char* name,unsigned long & pos,string & err)
{
  if (pos==0xffffffff)
    { err="get_*: position is 0xffffffff"; return 0; };
  if (!c[pos])
    { err="get_*: EOF on position"; return 0; };
  
  unsigned long tpos=gethex(c+pos+1);
  bool found=0;
  pos+=10;
  tpos+=33;
  while (t[tpos]!=' ')
    {
      if (!strncmp(t+tpos,name,strlen(name)) &&
          t[tpos+strlen(name)]==' ')
        {
          found=1;
          break;
        };
      pos+=10;
      tpos+=33;
    };
  if (!found)
    { err="get_*: "+(string) name+" not found"; return 0; };
  return 1;
}

char* stringelem(char* c,char* w,char* t,
		 char* name,unsigned long pos,string & err)
{
  err=0;
  if (!getpos(c,w,t,name,pos,err)) return 0;
  if (c[pos]!='s')
    { err="get_stringelem: "+(string) name+" has wrong type"; return 0; };

  unsigned long number=gethex(c+pos+1);
  if (number==0xffffffff) return 0; /* undefined string */
  unsigned long len=gethex(w+number);
  char* s=(char*) malloc(len+1);
  if (!s) log(LOG_FATAL,__FILE__,__LINE__,ERROR_OUT_OF_MEM,0);
  strncpy(s,w+number+8,len);
  s[len]=0;
  return s;
}

unsigned long classelem(char* c,char* w,char* t,
			char* name,unsigned long pos,string & err)
{
  err=0;
  if (!getpos(c,w,t,name,pos,err)) return 0;
  if (c[pos]!='c' && c[pos]!='p')
    { err="get_classelem: "+(string) name+" has wrong type"; return 0; };
  return gethex(c+pos+1);
}

unsigned long ulongelem(char* c,char* w,char* t,
                        char* name,unsigned long pos,string & err)
{
  err=0;
  if (!getpos(c,w,t,name,pos,err)) return 0;
  if (c[pos]!='u')
    { err="get_ulongelem: "+(string) name+" has wrong type"; return 0; };
  return gethex(c+pos+1);
}

long longelem(char* c,char* w,char* t,
                        char* name,unsigned long pos,string & err)
{
  err=0;
  if (!getpos(c,w,t,name,pos,err)) return 0;
  if (c[pos]!='l')
    { err="get_longelem: "+(string) name+" has wrong type"; return 0; };
  unsigned long number=gethex(c+pos+1);
  return (long) number;
}

unsigned short ushortelem(char* c,char* w,char* t,
			  char* name,unsigned long pos,string & err)
{
  err=0;
  if (!getpos(c,w,t,name,pos,err)) return 0;
  if (c[pos]!='n')
    { err="get_ushortelem: "+(string) name+" has wrong type"; return 0; };
  return (unsigned short) gethex(c+pos+1);
}

short shortelem(char* c,char* w,char* t,
                          char* name,unsigned long pos,string & err)
{
  err=0;
  if (!getpos(c,w,t,name,pos,err)) return 0;
  if (c[pos]!='o')
    { err="get_shortelem: "+(string) name+" has wrong type"; return 0; };
  return (short) gethex(c+pos+1);
}
